@@ -39,6 +39,7 @@ from ..bili.live import Live
 from ..bili.typing import QualityNumber
 from ..flv.stream_processor import StreamProcessor, BaseOutputFileManager
 from ..utils.mixins import AsyncCooperationMix, AsyncStoppableMixin
+from ..path import escape_path
 from ..flv.exceptions import FlvStreamCorruptedError
 from ..bili.exceptions import (
     LiveRoomHidden, LiveRoomLocked, LiveRoomEncrypted, NoStreamUrlAvailable
@@ -175,6 +176,10 @@ class StreamRecorder(
         if self._stream_processor is not None:
             self._stream_processor.duration_limit = value
 
+    @property
+    def recording_path(self) -> Optional[str]:
+        return self._file_manager.curr_path
+
     def has_file(self) -> bool:
         return self._file_manager.has_file()
 
@@ -184,6 +189,16 @@ class StreamRecorder(
     def clear_files(self) -> None:
         self._file_manager.clear_files()
 
+    def can_cut_stream(self) -> bool:
+        if self._stream_processor is None:
+            return False
+        return self._stream_processor.can_cut_stream()
+
+    def cut_stream(self) -> bool:
+        if self._stream_processor is None:
+            return False
+        return self._stream_processor.cut_stream()
+
     def update_progress_bar_info(self) -> None:
         if self._progress_bar is not None:
             self._progress_bar.set_postfix_str(self._make_pbar_postfix())
@@ -303,6 +318,7 @@ class StreamRecorder(
                 logger.warning(repr(e))
             except FlvStreamCorruptedError as e:
                 logger.warning(repr(e))
+                url = self._get_live_stream_url()
 
     def _streaming(self, url: str) -> None:
         logger.debug('Getting the live stream...')
@@ -315,6 +331,9 @@ class StreamRecorder(
             logger.debug('Response received')
             response.raise_for_status()
 
+            if self._stopped:
+                return
+
             assert self._stream_processor is not None
             self._stream_processor.process_stream(
                 io.BufferedReader(
@@ -486,10 +505,10 @@ class OutputFileManager(BaseOutputFileManager, AsyncCooperationMix):
         date_time = datetime.fromtimestamp(self._start_time)
         relpath = self.path_template.format(
             roomid=self._live.room_id,
-            uname=self._live.user_info.name,
-            title=self._live.room_info.title,
-            area=self._live.room_info.area_name,
-            parent_area=self._live.room_info.parent_area_name,
+            uname=escape_path(self._live.user_info.name),
+            title=escape_path(self._live.room_info.title),
+            area=escape_path(self._live.room_info.area_name),
+            parent_area=escape_path(self._live.room_info.parent_area_name),
             year=date_time.year,
             month=str(date_time.month).rjust(2, '0'),
             day=str(date_time.day).rjust(2, '0'),
