@@ -2,7 +2,9 @@ from .helpers import (
     file_exists,
     create_file,
     danmaku_path,
+    raw_danmaku_path,
     extra_metadata_path,
+    escape_path,
 )
 
 
@@ -10,5 +12,7 @@ __all__ = (
     'file_exists',
     'create_file',
     'danmaku_path',
+    'raw_danmaku_path',
     'extra_metadata_path',
+    'escape_path',
 )
