@@ -1,3 +1,4 @@
+import re
 import os
 from pathlib import PurePath
 
@@ -6,7 +7,9 @@ __all__ = (
     'file_exists',
     'create_file',
     'danmaku_path',
+    'raw_danmaku_path',
     'extra_metadata_path',
+    'escape_path',
 )
 
 
@@ -24,5 +27,13 @@ def danmaku_path(video_path: str) -> str:
     return str(PurePath(video_path).with_suffix('.xml'))
 
 
+def raw_danmaku_path(video_path: str) -> str:
+    return str(PurePath(video_path).with_suffix('.jsonl'))
+
+
 def extra_metadata_path(video_path: str) -> str:
     return video_path + '.meta.json'
+
+
+def escape_path(path: str) -> str:
+    return re.sub(r'[\\/:*?"<>|]', '', path)
